@@ -40,6 +40,19 @@
 //! * send evicted items to a channel, achieving the equivalent to an eviction listener feature.
 //! * zero out item weights so they are left in the cache instead of evicted.
 //!
+//! # Approximate memory usage
+//!
+//! The memory overhead per entry is `21` bytes.
+//!
+//! The memory usage of the cache data structures can be estimated as:
+//! `(size_of::<K>() + size_of::<V>() + 21) * (length * 1.5).next_power_of_two()`
+//!
+//! Actual memory usage may vary depending on the cache options and the key and value types, which can have external
+//! allocations (e.g. `String`, `Vec`, etc.). The above formula only accounts for the cache's data structures.
+//!
+//! The `1.5` value in the formula above results from `1 + G`, where `G` is the configured ghost allocation specified
+//! in [`OptionsBuilder::ghost_allocation`], which is `0.5` by default.
+//!
 //! # Hasher
 //!
 //! By default the crate uses [ahash](https://crates.io/crates/ahash), which is enabled (by default) via
@@ -51,8 +64,17 @@
 //! By default the crate uses [parking_lot](https://crates.io/crates/parking_lot), which is enabled (by default) via
 //! a crate feature with the same name. If the `parking_lot` feature is disabled the crate defaults to the std lib
 //! implementation instead.
+//!
+//! # Cargo Features
+//!
+//! | Feature | Default | Description |
+//! |---------|---------|-------------|
+//! | `ahash` | ✓ | Use [ahash](https://crates.io/crates/ahash) as the default hasher. When disabled, falls back to std lib's `RandomState` (currently SipHash-1-3). |
+//! | `parking_lot` | ✓ | Use [parking_lot](https://crates.io/crates/parking_lot) for synchronization primitives. When disabled, falls back to std lib's `RwLock`. |
+//! | `shuttle` | | Enable [shuttle](https://crates.io/crates/shuttle) testing support for concurrency testing. |
+//! | `stats` | | Enable cache statistics tracking via the `hits()` and `misses()` methods. |
 #![allow(clippy::type_complexity)]
-#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 #[cfg(not(fuzzing))]
 mod linked_slab;
@@ -177,6 +199,22 @@ pub trait Lifecycle<Key, Val> {
     fn end_request(&self, state: Self::RequestState) {}
 }
 
+/// The memory used by the cache
+///
+/// This struct exposes some implementation details, may change in the future
+#[non_exhaustive]
+#[derive(Debug, Copy, Clone)]
+pub struct MemoryUsed {
+    pub entries: usize,
+    pub map: usize,
+}
+
+impl MemoryUsed {
+    pub fn total(&self) -> usize {
+        self.entries + self.map
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use std::{
