@@ -9,7 +9,7 @@ use hashbrown::HashTable;
 use crate::{
     linked_slab::{LinkedSlab, Token},
     shim::sync::atomic::{self, AtomicU16},
-    Equivalent, Lifecycle, Weighter,
+    Equivalent, Lifecycle, MemoryUsed, Weighter,
 };
 
 #[cfg(feature = "stats")]
@@ -211,6 +211,13 @@ impl<Key, Val, We, B, L, Plh: SharedPlaceholder> CacheShard<Key, Val, We, B, L,
 }
 
 impl<Key, Val, We, B, L, Plh> CacheShard<Key, Val, We, B, L, Plh> {
+    pub fn memory_used(&self) -> MemoryUsed {
+        MemoryUsed {
+            entries: self.entries.memory_used(),
+            map: self.map.allocation_size(),
+        }
+    }
+
     pub fn weight(&self) -> u64 {
         self.weight_hot + self.weight_cold
     }
@@ -597,6 +604,19 @@ impl<
         self.remove_internal(hash, idx)
     }
 
+    pub fn remove_if<Q, F>(&mut self, hash: u64, key: &Q, f: F) -> Option<(Key, Val)>
+    where
+        Q: Hash + Equivalent<Key> + ?Sized,
+        F: FnOnce(&Val) -> bool,
+    {
+        let (idx, resident) = self.search_resident(hash, key)?;
+        if f(&resident.value) {
+            self.remove_internal(hash, idx)
+        } else {
+            None
+        }
+    }
+
     pub fn remove_token(&mut self, token: Token) -> Option<(Key, Val)> {
         let Some((Entry::Resident(resident), _)) = self.entries.get(token) else {
             return None;
@@ -731,8 +751,13 @@ impl<
                 *resident.referenced.get_mut() = (*resident.referenced.get_mut())
                     .min(MAX_F)
                     .saturating_sub(1);
-                if unpinned == 0 && Some(next) == self.hot_head {
-                    return false;
+                if Some(next) == self.hot_head {
+                    if unpinned == 0 {
+                        // All entries are pinned
+                        return false;
+                    }
+                    // Restart unpinned count
+                    unpinned = 0;
                 }
                 idx = next;
                 continue;
@@ -809,7 +834,7 @@ impl<
                 enter_state = ResidentState::Hot;
             }
             Entry::Placeholder(ph) => {
-                referenced = 1; // Pretend it's a newly insereted Resident
+                referenced = 1; // Pretend it's a newly inserted Resident
                 enter_state = ph.hot;
             }
         }
@@ -926,12 +951,14 @@ impl<
         if weight > self.weight_target_hot && !self.lifecycle.is_pinned(&key, &value) {
             self.lifecycle.before_evict(lcs, &key, &mut value);
             weight = self.weighter.weight(&key, &value);
+            // check again, it could have changed weight
             if weight > self.weight_target_hot {
                 return self.handle_overweight_replace_placeholder(lcs, placeholder, key, value);
             }
         }
 
-        if self.weight_hot + self.weight_cold + weight <= self.weight_capacity {
+        // cache is filling up, admit as hot if possible
+        if self.weight_hot + weight <= self.weight_target_hot {
             placeholder_hot = ResidentState::Hot;
         }
         *entry = Entry::Resident(Resident {
@@ -988,6 +1015,7 @@ impl<
         if weight > self.weight_target_hot && !self.lifecycle.is_pinned(&key, &value) {
             self.lifecycle.before_evict(lcs, &key, &mut value);
             weight = self.weighter.weight(&key, &value);
+            // check again, it could have changed weight
             if weight > self.weight_target_hot {
                 return self.handle_insert_overweight(lcs, hash, key, value, strategy);
             }
@@ -1106,6 +1134,28 @@ impl<
         record_miss_mut!(self);
         Err((shared, true))
     }
+
+    pub fn set_capacity(&mut self, new_weight_capacity: u64) {
+        // Calculate the ratio to proportionally adjust ghost capacity
+        let old_new_ratio = new_weight_capacity as f64 / self.weight_capacity as f64;
+        let hot_ratio = self.weight_target_hot as f64 / self.weight_capacity as f64;
+
+        // Update capacities
+        self.weight_capacity = new_weight_capacity;
+        self.weight_target_hot = (new_weight_capacity as f64 * hot_ratio) as u64;
+        self.capacity_non_resident = (self.capacity_non_resident as f64 * old_new_ratio) as usize;
+
+        // Evict items if we're over the new capacity
+        let mut lcs = self.lifecycle.begin_request();
+        while self.weight_hot + self.weight_cold > self.weight_capacity
+            && self.advance_cold(&mut lcs)
+        {}
+        self.lifecycle.end_request(lcs);
+        // Trim ghost entries if needed
+        while self.num_non_resident > self.capacity_non_resident {
+            self.advance_ghost();
+        }
+    }
 }
 
 /// Structure wrapping a mutable reference to a cached item.
