@@ -2,7 +2,7 @@ use crate::{
     linked_slab::Token,
     options::*,
     shard::{self, CacheShard, InsertStrategy},
-    DefaultHashBuilder, Equivalent, Lifecycle, UnitWeighter, Weighter,
+    DefaultHashBuilder, Equivalent, Lifecycle, MemoryUsed, UnitWeighter, Weighter,
 };
 use std::hash::{BuildHasher, Hash};
 
@@ -159,7 +159,7 @@ impl<Key: Eq + Hash, Val, We: Weighter<Key, Val>, B: BuildHasher, L: Lifecycle<K
 
     /// Fetches an item from the cache.
     ///
-    /// Note: Leaking the returned RefMut might cause undefined behavior.
+    /// Note: Leaking the returned RefMut might cause cache weight tracking to be inaccurate.
     pub fn get_mut<Q>(&mut self, key: &Q) -> Option<RefMut<'_, Key, Val, We, B, L>>
     where
         Q: Hash + Equivalent<Key> + ?Sized,
@@ -177,7 +177,7 @@ impl<Key: Eq + Hash, Val, We: Weighter<Key, Val>, B: BuildHasher, L: Lifecycle<K
 
     /// Peeks an item from the cache. Contrary to gets, peeks don't alter the key "hotness".
     ///
-    /// Note: Leaking the returned RefMut might cause undefined behavior.
+    /// Note: Leaking the returned RefMut might cause cache weight tracking to be inaccurate.
     pub fn peek_mut<Q>(&mut self, key: &Q) -> Option<RefMut<'_, Key, Val, We, B, L>>
     where
         Q: Hash + Equivalent<Key> + ?Sized,
@@ -194,6 +194,18 @@ impl<Key: Eq + Hash, Val, We: Weighter<Key, Val>, B: BuildHasher, L: Lifecycle<K
         self.shard.remove(self.shard.hash(key), key)
     }
 
+    /// Remove an item from the cache whose key is `key` if `f(&value)` returns `true` for that entry.
+    /// Compared to peek and remove, this method is more efficient as it requires only 1 lookup.
+    ///
+    /// Returns the removed entry, if any.
+    pub fn remove_if<Q, F>(&mut self, key: &Q, f: F) -> Option<(Key, Val)>
+    where
+        Q: Hash + Equivalent<Key> + ?Sized,
+        F: FnOnce(&Val) -> bool,
+    {
+        self.shard.remove_if(self.shard.hash(key), key, f)
+    }
+
     /// Replaces an item in the cache, but only if it already exists.
     /// If `soft` is set, the replace operation won't affect the "hotness" of the key,
     /// even if the value is replaced.
@@ -315,6 +327,8 @@ impl<Key: Eq + Hash, Val, We: Weighter<Key, Val>, B: BuildHasher, L: Lifecycle<K
     /// Gets an item from the cache with key `key` .
     /// If the corresponding value isn't present in the cache, this functions returns a guard
     /// that can be used to insert the value once it's computed.
+    ///
+    /// Note: Leaking the returned RefMut might cause cache weight tracking to be inaccurate.
     pub fn get_mut_or_guard<'a, Q>(
         &'a mut self,
         key: &Q,
@@ -373,10 +387,26 @@ impl<Key: Eq + Hash, Val, We: Weighter<Key, Val>, B: BuildHasher, L: Lifecycle<K
         self.shard.drain()
     }
 
+    /// Sets the cache to a new weight capacity.
+    ///
+    /// If the new capacity is smaller than the current weight, items will be evicted
+    /// to bring the cache within the new limit.
+    pub fn set_capacity(&mut self, new_weight_capacity: u64) {
+        self.shard.set_capacity(new_weight_capacity);
+    }
+
     #[cfg(any(fuzzing, test))]
     pub fn validate(&self, accept_overweight: bool) {
         self.shard.validate(accept_overweight);
     }
+
+    /// Get total memory used by cache data structures
+    ///
+    /// It should be noted that if cache key or value is some type like `Vec<T>`,
+    /// the memory allocated in the heap will not be counted.
+    pub fn memory_used(&self) -> MemoryUsed {
+        self.shard.memory_used()
+    }
 }
 
 impl<Key, Val, We, B, L> std::fmt::Debug for Cache<Key, Val, We, B, L> {
@@ -560,4 +590,87 @@ mod tests {
         assert_eq!(cache.get(&0).copied(), Some(0));
         assert!(cache.contains_key(&0));
     }
+
+    #[test]
+    fn test_set_capacity() {
+        let mut cache = Cache::new(100);
+        for i in 0..80 {
+            cache.insert(i, i);
+        }
+        let initial_len = cache.len();
+        assert!(initial_len <= 80);
+
+        // Set to smaller capacity
+        cache.set_capacity(50);
+        assert!(cache.len() <= 50);
+        assert!(cache.weight() <= 50);
+        cache.validate(false);
+
+        // Set to larger capacity
+        cache.set_capacity(200);
+        assert_eq!(cache.capacity(), 200);
+        cache.validate(false);
+
+        // Insert more items
+        for i in 100..180 {
+            cache.insert(i, i);
+        }
+        assert!(cache.len() <= 180);
+        assert!(cache.weight() <= 200);
+        cache.validate(false);
+    }
+
+    #[test]
+    fn test_set_capacity_with_ghosts() {
+        // Create a cache that will generate ghost entries
+        let mut cache = Cache::new(50);
+
+        // Insert items to fill the cache
+        for i in 0..100 {
+            cache.insert(i, i);
+        }
+        cache.validate(false);
+
+        // Set to smaller capacity - should trim both resident and ghost entries
+        cache.set_capacity(25);
+        assert!(cache.weight() <= 25);
+        cache.validate(false);
+
+        // Set back to larger capacity
+        cache.set_capacity(100);
+        assert_eq!(cache.capacity(), 100);
+        cache.validate(false);
+
+        // Insert more items
+        for i in 100..150 {
+            cache.insert(i, i);
+        }
+        cache.validate(false);
+    }
+
+    #[test]
+    fn test_remove_if() {
+        let mut cache = Cache::new(100);
+
+        // Insert test data
+        cache.insert(1, 10);
+        cache.insert(2, 20);
+        cache.insert(3, 30);
+
+        // Test removing with predicate that returns true
+        let removed = cache.remove_if(&2, |v| *v == 20);
+        assert_eq!(removed, Some((2, 20)));
+        assert_eq!(cache.get(&2), None);
+
+        // Test removing with predicate that returns false
+        let not_removed = cache.remove_if(&3, |v| *v == 999);
+        assert_eq!(not_removed, None);
+        assert_eq!(cache.get(&3), Some(&30));
+
+        // Test removing non-existent key
+        let not_found = cache.remove_if(&999, |_| true);
+        assert_eq!(not_found, None);
+
+        cache.validate(false);
+    }
 }
